using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;

namespace WebFormMCalc
{
	/// <summary>
	/// Class Model.cs
	/// jlouie 07.07.02
	/// Adapted from Model.java
	/// "Visual Cafe for Java Explorer, Database Development Edition"
	/// William Brogden, Jeffrey A. Louie, and Ed Tittel, Coriolis, 1998, 585pp.
	/// Supplied "as is"
	/// No warranty is expressed or implied
	/// This code is for instructional use only
	/// </summary>
	public class Model 
	{
		// internal class constants, not "versionable"
		private const int INVALID= -1;  // flags error
		private const int PRINCIPAL= 0;
		private const int INTEREST= 1;
		private const int MONTHS= 2;
		private const int PAYMENT= 3;

		private double[] arrayDb= new double[4];
		private int target= INVALID;
		private string message= "";

		/* // uncomment to run console self test
		// self test static method outputs state to console
		static void ConsoleDebug(Model model) 
		{
			if (model == null) 
			{
				System.Console.WriteLine("Null object.");
				return;
			}
			System.Console.WriteLine("Message: "+model.Message);
			System.Console.WriteLine("Result: "+model.Result);
			System.Console.WriteLine("Principal: "+model.Principal);
			System.Console.WriteLine("Interest: "+model.Interest);
			System.Console.WriteLine("Months: "+model.Months);
			System.Console.WriteLine("Payment: "+model.Payment);
		}
		*/

		/* // uncomment to run console self test
		// self test
		[STAThread]
		static void Main() 
		{ 
			// test internal consistency of algorithms
			Model model= new Model(100000,8.5,360,0);
			Model.ConsoleDebug(model); // payment = 768.9134584334
			model.Init(0,8.5,360,768.9134584334);
			Model.ConsoleDebug(model);
			model.Init(100000,0,360,768.9134584334);
			Model.ConsoleDebug(model);
			model.Init(100000,8.5,0,768.9134584334);
			Model.ConsoleDebug(model);
			System.Console.ReadLine();
		}*/
	 
		// no arg constructor
		public Model(){;}

		// arg constructor
		public Model(double principal, double interest, int months, double payment) 
		{
			Init(principal, interest, months, payment);
		}

		// factored code, can be called after call to constructor
		// allowing reuse of instance of class
		// eg. object is _not_ immutable by design
		public void Init(double principal, double interest, int months, double payment) 
		{
			// reset flags
			target= INVALID;
			message= "";

			// store input into array of double
			arrayDb[PRINCIPAL]= principal;
			arrayDb[INTEREST]= interest;
			arrayDb[MONTHS]= (double)months;
			arrayDb[PAYMENT]= payment;

			// validate input

			// one, and only one, "value" must be zero --> target
			int zeros= 0;
			int tempTarget= INVALID;
			for (int i=0; i<4; i++) 
			{
				if (arrayDb[i] == 0) 
				{
					zeros++;
					tempTarget= i;
				}
			}
			if (zeros>1) 
			{
				message= "Too many zero parameters.";
				return;
			}
			if (zeros == 0) 
			{
				message= "One parameter must be zero.";
				return;
			}

			// validate interest
			if (interest > 100 || interest < 0) 
			{
				message= "Invalid interest.";
				return;
			}

			// validate months
			if (months < 0) 
			{
				message= "Invalid months.";
				return;
			}

			// validate principal
			if (principal < 0) 
			{
				message= "Invalid principal.";
				return;
			}

			// validate payment
			if (payment < 0) 
			{
				message= "Invalid payment.";
				return;
			}

			// input parameters appear valid
			target= tempTarget;
			DoAlgorithm(target);
		}

		// the actual amortization algorithm
		// m= P*i(1-(1+i)^-N)
		// i=r/1200
		// result= 0 --> marks error
		private void DoAlgorithm(int target) 
		{
			double result= 0;
			double P= arrayDb[PRINCIPAL];  // principal
			double i= arrayDb[INTEREST]/1200; // monthly percentage rate
			double N= arrayDb[MONTHS]; // loan period in months
			double m= arrayDb[PAYMENT]; // monthly payment
			
			if (target>= 0 && target< 4) // validate target
			{
				try 
				{
					switch (target) 
					{
						case PRINCIPAL:  // principal
							result= 1+i;
							result= 1/Math.Pow(result, N);
							result= ((1-result)/i)*m;
							break;
						case INTEREST:  // annual interest
							// algorithm fails if N*m >= P !!
							if ((N*m)<P) 
							{
								throw new ArithmeticException();
							}
							// factor out Interest function, too long
							result= CalcInterest(P,N,m);
							break;
						case MONTHS:  // loan period
							result= (1-(P*i/m));
							result= Math.Log(result);
							result= -result/Math.Log((1+i));
							break;
						case PAYMENT:  // monthly payments
							result= 1+i;
							result= 1/Math.Pow(result,N);
							result= (P*i)/(1-result);
							break;
							//default:
							//break;
					}
				}
				catch 
				{
					result= 0;
				}
			}

			// validate result
			if (Double.IsNaN(result)) 
			{
				result= 0;
			}
			if (result == 0) 
			{
				message= "Input Error.";
			}
			else // valid result
			{
				arrayDb[target]= result;
			}
		}
		
		// a complex iterative calculation for interest
		// thanks to Dr. W. Carlini (and Newton)for the solution
		// returns zero on error
		// ASSERT (N*m)>=P
		private double CalcInterest(double P, double N, double m) 
		{
			double temp= (m/P), answer= (m/P), diff= 100, numerator= 0, denominator= 0, accuracy= .00001;
			int index, maxIterations= 1000;

			try 
			{
				for (index= 0; ((diff>accuracy) && (index<maxIterations)); index++) 
				{
					temp= answer;
					numerator= (P*temp/m)+Math.Pow((1+temp),-N)-1;
					denominator= (P/m)-N*Math.Pow((1+temp),(-N-1));
					// if (denominator ==0 ){throw new ArithmeticException();}
					answer= temp-(numerator/denominator);
					diff= answer- temp;
					if (diff<0) 
					{
						diff= -diff;
					}
				}
				answer *= 1200;

				// validate answer
				if ((answer<0) || Double.IsNaN(answer) || (index == maxIterations)) 
				{
					throw new ArithmeticException();
				}
			}
			catch 
			{
				answer= 0;
			}
			return answer;
		}

		// default target is -1 (INVALID)
		public bool IsValid() 
		{
			return ((target>=PRINCIPAL) && (target<=PAYMENT)) ? true: false;
		}

		// Java "getter" code converted to C# get only properties
		public double Result 
		{
			get 
			{
				if (IsValid()) 
				{
					return arrayDb[target];
				}
				else 
				{
					return 0.0;
				}
			}
		}
		public int Target 
		{
			get 
			{
				return target;
			}
		}
		public string Message 
		{
			get 
			{
				return message;
			}
		}
		public double Principal
		{
			get 
			{
				if (IsValid()) 
				{
					return arrayDb[PRINCIPAL];
				}
				else 
				{
					return 0.0;
				}
			}
		}
		public double Interest 
		{
			get 
			{
				if (IsValid()) 
				{
					return arrayDb[INTEREST];
				}
				else 
				{
					return 0.0;
				}
			}
		}
		public double Months
		{
			get 
			{
				if (IsValid()) 
				{
					return arrayDb[MONTHS];
				}
				else 
				{
					return 0;
				}
			}
		}
		public double Payment 
		{
			get 
			{
				if (IsValid()) 
				{
					return arrayDb[PAYMENT];
				}
				else 
				{
					return 0.0;
				}
			}
		}
	}
	/// <summary>
	/// Summary description for WebForm1.
	/// </summary>
	public class WebForm1 : System.Web.UI.Page
	{
		protected System.Web.UI.WebControls.Label Label1;
		protected System.Web.UI.WebControls.Label Label2;
		protected System.Web.UI.WebControls.Label Label3;
		protected System.Web.UI.WebControls.Label Label4;
		protected System.Web.UI.WebControls.Button buttonCalc;
		protected System.Web.UI.WebControls.Button buttonReset;
		protected System.Web.UI.WebControls.Label Label6;
		protected System.Web.UI.WebControls.Label Label5;
		protected System.Web.UI.WebControls.TextBox textBoxPrincipal;
		protected System.Web.UI.WebControls.TextBox textBoxInterest;
		protected System.Web.UI.WebControls.TextBox textBoxPeriod;
		protected System.Web.UI.WebControls.TextBox textBoxPayment;
		protected System.Web.UI.WebControls.TextBox textBoxMessage;
		protected System.Web.UI.WebControls.Label Label7;

		private Model model= new Model();
	
		private void Page_Load(object sender, System.EventArgs e)
		{
			// Put user code to initialize the page here		
		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.buttonCalc.Click += new System.EventHandler(this.buttonCalc_Click);
			this.buttonReset.Click += new System.EventHandler(this.buttonReset_Click);
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion

		private void buttonCalc_Click(object sender, System.EventArgs e)
		{
			double principal= 0;
			double interest= 0;
			int months= 0;
			double payment= 0;
			bool isInputError= false;

			// validate user input, must allow zero
			try 
			{
				principal= Double.Parse(textBoxPrincipal.Text);
				if (principal<0) 
				{
					throw new Exception();
				}
			}
			catch 
			{
				textBoxPrincipal.Text= "Invalid Input.";
				isInputError= true;
			}
			try 
			{
				interest= Double.Parse(textBoxInterest.Text);
				if ((interest < 0) || (interest > 100)) 
				{
					throw new Exception();
				}
			}
			catch 
			{
				textBoxInterest.Text= "Invalid Input.";
				isInputError= true;
			}
			try 
			{
				months= Int32.Parse(textBoxPeriod.Text);
				if (months <0)
				{
					throw new Exception();
				}
			}
			catch 
			{
				textBoxPeriod.Text= "Invalid Input.";
				isInputError= true;
			}
			try 
			{
				payment= Double.Parse(textBoxPayment.Text);
				if (payment < 0) 
				{
					throw new Exception();
				}
			}
			catch 
			{
				textBoxPayment.Text= "Invalid Input.";
				isInputError= true;
			}
			if (isInputError) 
			{
				return;
			}

			// valid user input
			model.Init(principal,interest,months,payment);
			if (model.IsValid()) 
			{
				textBoxPrincipal.Text= model.Principal.ToString();
				textBoxInterest.Text= model.Interest.ToString();
				textBoxPeriod.Text= model.Months.ToString();
				textBoxPayment.Text= model.Payment.ToString();
				textBoxMessage.Text= model.Message.ToString();
			}
			else 
			{
				ResetControls();
				textBoxMessage.Text= model.Message.ToString();
			}
		}

		private void buttonReset_Click(object sender, System.EventArgs e)
		{
			ResetControls();
			textBoxMessage.Text= "";
		}

		private void ResetControls() 
		{
			textBoxPrincipal.Text= "";
			textBoxInterest.Text= "";
			textBoxPeriod.Text= "";
			textBoxPayment.Text= "0";
		}
	}
}
